##    GNU AGPLv3
##
##    dirock reflow software
##    Copyright (C) 2014  dirock UG (haftungsbeschraenkt)
##
##    This program is free software: you can redistribute it and/or modify
##    it under the terms of the GNU Affero General Public License as published by
##    the Free Software Foundation, either version 3 of the License, or
##    (at your option) any later version.
##
##    This program is distributed in the hope that it will be useful,
##    but WITHOUT ANY WARRANTY; without even the implied warranty of
##    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##    GNU Affero General Public License for more details.
##
##    You should have received a copy of the GNU Affero General Public License
##    along with this program.  If not, see www.gnu.org/licenses/.
##    Lars.Rockstroh@dirock.de

## . When you press "Start x", stage x will start from the beginning.
## . When the target temperature of a stage is reached but the time is not up,
##   power will toogle between 1 percent and 100 percent to fit target temperature.
## . The next state is entered, when time is up and the temperature was at any time in this stage at least the target temperature.
## . You can change temperature profile values at any time but the program accounts the updates only when you press a button.
## . There is no "delta temperature/time" mechanism. Use more stages to emulate.
## . Sampling rate = 1 Hz due to controller
## . Don't forget to adjust the sensor tempoffset

from PySide.QtGui import *
from PySide.QtCore import *
from user_form import *
from numpy import *
import time
import datetime
import serial
import sys



class PlainSignal_class(QObject):
    sig = Signal()
class IntSignal_class(QObject):
    sig = Signal(int)
class StrSignal_class(QObject):
    sig = Signal(str)
    
class connthread_class(QThread):             #reading in own thread, rest in gui thread
    def __init__(self, parent = None):
        QThread.__init__(self, parent)

        self.signal_updatevisual = IntSignal_class()
        self.signal_save = PlainSignal_class()
        self.signal_statusTextEdit = StrSignal_class()
        
        self.data=zeros((3,5000),'int32')  #maximum logging of 5000 samples
        self.sample_number=0
        self.stage=7
        self.profiletable=[]
        self.power=1
        self.temp_last=0
        self.temp_set=0
        self.logging_interval=1
        self.logging_interval_count=0

        self.conn_opened=0       
        self.ser=serial.Serial()
        self.ser.baudrate=9600
        self.comport='12'
        self.ser.timeout=1; self.ser.writeTimeout=10; self.ser.rtscts=0#1

    def connection(self, make_open):
        if not(make_open) and self.conn_opened:
            self.ser.close()
            self.signal_statusTextEdit.sig.emit("Serial port closed")
            self.conn_opened=make_open
        if make_open and not(self.conn_opened):
            try:
                self.ser.port=int(self.comport)
            except:
                self.signal_statusTextEdit.sig.emit("Please check Comport Settings, only numbers please")
                return False
            try:
                self.ser.open()
                self.signal_statusTextEdit.sig.emit("Serial port opened")
            except serial.serialutil.SerialException:
                self.signal_statusTextEdit.sig.emit("Opening serial port failed. Please check Cable connection and comport settings (only numbers please)")
                return False
            self.ser.write('manual 1\n')
            self.flush()
            self.conn_opened=make_open
        return True
    
    def flush(self):
        QtCore.QThread.msleep(10)
        while self.ser.inWaiting():
            self.ser.read(self.ser.inWaiting())
            QtCore.QThread.msleep(10)
    def shot(self,value):
        self.ser.write('shot '+str(value)+'\n')
    def read_after_shot(self):
        while not(self.ser.inWaiting()):
            QtCore.QThread.msleep(10)
        while True:
            a=self.ser.readline().split(', ')
            if a[0]=='shot' and a[3]==' degC\r\n':
                self.flush()
                return int(a[2])
            else:
                QtCore.QThread.msleep(10)
                

    def change_stage(self,stage,profiletable):
        if profiletable: #GUI calling -> copy profile
            self.profiletable=profiletable
        self.timestamp_timeleft=time.time()+self.profiletable[stage][0]
        self.temp_set=self.profiletable[stage][1]

        if self.stage!=7 and stage==7:
            self.signal_save.sig.emit()
        if self.stage==7 and stage!=7:
            self.start() #start the Reading-Process

        self.stage=stage
        self.temp_was_hit=False
        self.signal_updatevisual.sig.emit(1)
                
    def run(self): #the thread
        #doing initalization
        self.sample_number=0
        self.signal_updatevisual.sig.emit(1)
        self.timestamp_start=time.time()
        if not(self.connection(1)): #open com port
            self.change_stage(7,'')  #Error, back to Zero, which is 7
            return
        self.logging_interval_count=0
        self.shot(1); self.temp_last=self.read_after_shot() #to get initial temperature

        while self.stage!=7:
            if self.temp_last < self.temp_set:
                if (self.temp_set<230):
                    if self.temp_set-self.temp_last<2:
                        self.power=5
                    elif self.temp_set-self.temp_last<10:
                        self.power=25
                    elif self.temp_set-self.temp_last<30:
                        self.power=50
                    else:
                        self.power=100
                else:
                    self.power=100
            else:
                self.power=1
                self.temp_was_hit=True
            self.shot(self.power)
            # doing stuff here, while the controller is shoting and preparing his answer
            if not(self.logging_interval_count):
                self.data[0][self.sample_number]=int(time.time()-self.timestamp_start)
                self.data[1][self.sample_number]=self.temp_last
                self.data[2][self.sample_number]=self.temp_set
                self.sample_number+=1
            if (self.timestamp_timeleft < time.time()) and self.temp_was_hit: #time is up and temp_was_hit
                self.change_stage(self.stage+1,'')
            self.signal_updatevisual.sig.emit(not(self.logging_interval_count))
            self.logging_interval_count=(self.logging_interval_count+1)%self.logging_interval
            self.temp_last=self.read_after_shot()  # all done. Maybe controller can give temp now.
        self.connection(0) #close com port

class MyApplication(QtGui.QMainWindow, Ui_MainWindow):
    def __init__(self, parent=None):
        super(MyApplication, self).__init__(parent)
        self.setupUi(self)
        self.connthread=connthread_class()
        self.connthread.signal_updatevisual.sig.connect(self.updatevisual)
        self.connthread.signal_save.sig.connect(self.save)
        self.connthread.signal_statusTextEdit.sig.connect(self.update_statusTextEdit)

    def update_statusTextEdit(self,data):
        self.statusBar.showMessage(data)         

    def updatevisual(self, update_graph):
        if update_graph:
            sample_number=self.connthread.sample_number
            self.abcwidget.axes.plot(self.connthread.data[0][0:sample_number], self.connthread.data[1][0:sample_number],'g')
            self.abcwidget.draw()
        self.label_temp.setText(str(self.connthread.temp_last))
        self.label_stage.setText(str(self.connthread.stage))
        self.label_timeleft.setText(str(int(self.connthread.timestamp_timeleft-time.time())))
        self.label_power.setText(str(self.connthread.power))

    def slot_Button(self):
        stage=self.sender().text()[-1] #which button?
        if int(stage)!=7 and self.connthread.stage==7:
            self.groupBox_file.setEnabled(0)
            self.connthread.comport=self.lineEdit_com.text()  #Set com port
            self.connthread.logging_interval=int(self.box_samplinglog.value()) #Set logging intervall
        profiletable=[[eval('self.box_time'+str(k)+'.value()'),eval('self.box_temp'+str(k)+'.value()')] for k in range(8)] #collect data from gui
        self.connthread.change_stage(int(stage), profiletable)
        
    def save(self):  #GUI blocking; don't touch anything right now
        datetimestring=datetime.datetime.now().strftime("%y%m%d_%H%M_%S")
        self.abcwidget.figure.savefig(self.lineEdit_file.text()+'_'+datetimestring+'.png')
        sample_number=self.connthread.sample_number
        savetxt(self.lineEdit_file.text()+'_'+datetimestring+'.csv', self.connthread.data[:][:sample_number], fmt='%d',delimiter=",")
        self.groupBox_file.setEnabled(1)

if __name__ == "__main__":
    app = QtGui.QApplication(sys.argv)
    window = MyApplication()
    window.show()
    sys.exit(app.exec_())
